#!/usr/bin/env python
# Copyright (C) 2005-2011 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""A tool for uploading tarballs and installers of bzr to Launchpad."""

import os
import re
import subprocess
import sys

from launchpadlib.launchpad import Launchpad
from launchpadlib.uris import LPNET_SERVICE_ROOT


_bzr_version_re = re.compile(
    r'bzr-(?P<major>\d)\.(?P<minor>\d)'
    r'((?P<micro>(\.|b)\d))?'
    r'(-(?P<build>\d))?'
    r'(\.win32-py(?P<py_version>2\.\d))?')


def determine_bzr_version(name):
    """Extra the version information from the string."""
    m = _bzr_version_re.match(name)
    if not m:
        return None
    return m.groupdict()


def add_build_num(name, version_dict, build):
    """There isn't a build number, add it."""
    version_bits = '%s.%s' % (version_dict['major'], version_dict['minor'])
    if version_dict['micro'] is not None:
        # 'micro' is 'b1' or '.3' etc.
        version_bits += version_dict['micro']
    return name.replace(version_bits, '%s-%d' % (version_bits, build))


def find_sig_name(name):
    """See if a signature exists for this entry."""
    for ext in ('.asc', '.sig'):
        sig_name = name + ext
        if os.path.isfile(sig_name):
            return sig_name
    return None


def get_sig_name(name):
    """Find a signature file for this entry, or create one."""
    sig_name = find_sig_name(name)
    if sig_name is not None:
        return sig_name
    # If we got here, there is no signature yet
    ret = subprocess.call(['gpg', '--detach-sig', '--armor', name])
    if ret == 0:
        return name + '.asc'
    raise RuntimeError("failed to sign %s" % (name,))


def get_description(version_dict):
    """Get the description of this version."""
    py_version = version_dict['py_version']
    if py_version is None:
        return "Windows Standalone Installer"
    else:
        return "Windows Python %s Installer" % (py_version,)


def get_file_contents(filename):
    f = file(filename, "rb")
    try:
        return f.read()
    finally:
        f.close()
    

def main(args):
    import optparse
    p = optparse.OptionParser(usage='%prog [options] FILES')
    p.add_option('--verbose', '-v', action='store_true', help='Be chatty')
    p.add_option('--build', default=None, type=int,
                 help='Files that are missing a build id will be renamed'
                      ' to match this build num')
    p.add_option('--dry-run', default=False, action='store_true',
                 help='Don\'t actually run curl')

    (opts, args) = p.parse_args(args)
    files_to_upload = []
    for arg in args:
        if arg.endswith('.asc') or arg.endswith('.sig'):
            sys.stderr.write('Ignoring signature file: %s\n' % (arg,))
            continue
        version = determine_bzr_version(os.path.basename(arg))
        if version is None:
            sys.stderr.write('Unknown bzr version for %s\n' % (arg,))
            continue
        if version['build'] is None and opts.build is not None:
            build_name = add_build_num(arg, version, opts.build)
            os.rename(arg, build_name)
            sig_name = find_sig_name(arg)
            if sig_name is not None:
                build_sig_name = add_build_num(sig_name, version, opts.build)
                os.rename(sig_name, build_sig_name)
            arg = build_name
        sig_name = get_sig_name(arg)
        files_to_upload.append((arg, sig_name, version))
    # Now sort them
    def sort_key(info):
        version = info[-1]
        py_version = version['py_version']
        if py_version is None:
            # We want these to sort last
            py_version = '\xff\xff\xff\xff\xff\xff'
        return (version['major'], version['minor'], version['micro'],
                py_version)
    files_to_upload.sort(key=sort_key)
    lp = Launchpad.login_with("bzr-windows-installers",
        allow_access_levels=['WRITE_PRIVATE'],
        service_root=LPNET_SERVICE_ROOT)
    bzr = lp.projects["bzr"]
    releases = dict((rel.version, rel) for rel in bzr.releases)
    for f_name, sig_name, version_dict in files_to_upload:
        release_version = "%(major)s.%(minor)s%(micro)s" % version_dict
        try:
            rel = releases[release_version]
        except KeyError:
            sys.stderr.write("No release created for version %r from %r\n"
                % (release_version, f_name))
            sys.exit(1)
        sys.stderr.write('Uploading %s\n' % (f_name,))
        if not opts.dry_run:
            rel.add_file(
                filename=os.path.basename(f_name),
                description=get_description(version_dict),
                content_type="application/x-msdownload",
                file_type="Installer file",
                signature_filename=os.path.basename(sig_name),
                signature_content=get_file_contents(sig_name),
                # API not smart enough to stream, so can OOM for big files
                file_content=get_file_contents(f_name),
                )


if __name__ == '__main__':
    main(sys.argv[1:])

